﻿using Microsoft.AspNet.Identity;
using Microsoft.Owin.Security;
using SignetTokenTestWeb.Helpers;
using SignetTokenTestWeb.Models;
using SignetTokenTestWeb.SignetReference;
using System;
using System.Collections.Generic;
using System.Configuration;
using System.Linq;
using System.Security.Claims;
using System.Web;
using System.Web.Mvc;

namespace SignetTokenTestWeb.Controllers
{
    [Authorize]
    public class AuthedController : Controller
    {
        SignetHelper signet = new SignetHelper();
        string signeturl = ConfigurationManager.AppSettings.Get("SignetSignUrl");

        // GET: Authed
        [Authorize]
        public ActionResult Index()
        {
            ViewBag.phone = getClaimValue("mobilephone");
            ViewBag.signeturl = signeturl;
            return View();
        }

        [Authorize]
        public ActionResult Documents()
        {
            List<DocumentInfov2> docs = null;
            try
            {
                if (Session["token"] != null)
                    signet.SetToken(Session["token"].ToString());
                docs = signet.GetDocuments(getClaimValue("name"), getClaimValue("mobilephone"));
                if (docs != null)
                    Session["token"] = signet.GetToken();
            }
            catch (Exception ex)
            {
                ViewBag.message = ex.Message;
            }
            return PartialView(docs);
        }

        [Authorize]
        public ActionResult GetImages(string docid)
        {
            if (string.IsNullOrWhiteSpace(docid))
                return Json(new { Error = "No doc ID" }, JsonRequestBehavior.AllowGet);
            else
            {
                if (Session["token"] != null)
                    signet.SetToken(Session["token"].ToString());
                try
                {
                    var images = signet.GetImages(docid, getClaimValue("name"), getClaimValue("mobilephone"));
                    if (images != null)
                    {
                        Session["token"] = signet.GetToken();
                        JsonResult json = Json(new { images = images.Select(i => Convert.ToBase64String(i)).ToList() }, JsonRequestBehavior.AllowGet);
                        json.MaxJsonLength = int.MaxValue;
                        return json;
                    }
                    else
                        return Json(new { error = "No images" }, JsonRequestBehavior.AllowGet);
                }
                catch (Exception ex)
                {
                    return Json(new { error = ex.Message }, JsonRequestBehavior.AllowGet);
                }
            }
        }

        [Authorize]
        public ActionResult GetDocument(string docid)
        {
            if (string.IsNullOrWhiteSpace(docid))
                return Json(new { Error = "No doc ID" }, JsonRequestBehavior.AllowGet);
            else
            {
                if (Session["token"] != null)
                    signet.SetToken(Session["token"].ToString());
                try
                {
                    byte[] document = signet.GetDocument(docid, getClaimValue("name"), getClaimValue("mobilephone"));
                    if (document != null)
                    {
                        Session["token"] = signet.GetToken();
                        return Json(new { document = Convert.ToBase64String(document) }, JsonRequestBehavior.AllowGet);
                    }
                    else
                        return Json(new { error = "No document" }, JsonRequestBehavior.AllowGet);
                }
                catch (Exception ex)
                {
                    return Json(new { error = ex.Message }, JsonRequestBehavior.AllowGet);
                }
            }
        }

        [Authorize]
        public ActionResult GetContent(string docid)
        {
            if (string.IsNullOrWhiteSpace(docid))
                return Json(new { Error = "No doc ID" }, JsonRequestBehavior.AllowGet);
            else
            {
                if (Session["token"] != null)
                    signet.SetToken(Session["token"].ToString());
                try
                {
                    string content = signet.GetDocumentContent(docid, getClaimValue("name"), getClaimValue("mobilephone"));
                    if (content != null)
                    {
                        Session["token"] = signet.GetToken();
                        return Json(new { content = content }, JsonRequestBehavior.AllowGet);
                    }
                    else
                        return Json(new { error = "No document" }, JsonRequestBehavior.AllowGet);
                }
                catch (Exception ex)
                {
                    return Json(new { error = ex.Message }, JsonRequestBehavior.AllowGet);
                }
            }
        }

        [Authorize]
        public ActionResult GetDocumentInfo(string docid)
        {
            if (string.IsNullOrWhiteSpace(docid))
                return Json(new { Error = "No doc ID" }, JsonRequestBehavior.AllowGet);
            else
            {
                if (Session["token"] != null)
                    signet.SetToken(Session["token"].ToString());
                try
                {
                    DocumentInfo info = signet.GetDocumentInfo(docid, getClaimValue("name"), getClaimValue("mobilephone"));
                    if (info != null)
                    {
                        Session["token"] = signet.GetToken();
                        return Json(new { info = info }, JsonRequestBehavior.AllowGet);
                    }
                    else
                        return Json(new { error = "No document" }, JsonRequestBehavior.AllowGet);
                }
                catch (Exception ex)
                {
                    return Json(new { error = ex.Message }, JsonRequestBehavior.AllowGet);
                }
            }
        }

        [Authorize]
        public ActionResult SignDocument(string docid)
        {
            if (string.IsNullOrWhiteSpace(docid))
                return Json(new { Error = "No doc ID" }, JsonRequestBehavior.AllowGet);
            else
            {
                if (Session["token"] != null)
                    signet.SetToken(Session["token"].ToString());
                try
                {
                    if (string.IsNullOrWhiteSpace(getClaimValue("mobilephone")))
                    {
                        string token = signet.SignDocument(docid, getClaimValue("name"), getClaimValue("mobilephone"),
                            Request.Url.AbsoluteUri.ToLower().Replace("/signdocument", ""));
                        if (string.IsNullOrWhiteSpace(token))
                            return Json(new { error = "Could not sign" }, JsonRequestBehavior.AllowGet);
                        else
                        {
                            Session["token"] = signet.GetToken();
                            return Json(new { token = token, success = true }, JsonRequestBehavior.AllowGet);
                        }
                    }
                    else
                    {
                        bool success = signet.SignDocument(docid, getClaimValue("name"), getClaimValue("mobilephone"));
                        if (success)
                        {
                            Session["token"] = signet.GetToken();
                            return Json(new { success = true }, JsonRequestBehavior.AllowGet);
                        }
                        else
                            return Json(new { error = "Could not sign" }, JsonRequestBehavior.AllowGet);
                    }
                }
                catch (Exception ex)
                {
                    return Json(new { error = ex.Message }, JsonRequestBehavior.AllowGet);
                }
            }
        }

        [Authorize]
        public ActionResult RejectDocument(string docid, string reason)
        {
            if (string.IsNullOrWhiteSpace(docid))
                return Json(new { Error = "No doc ID" }, JsonRequestBehavior.AllowGet);
            else if (string.IsNullOrWhiteSpace(reason))
                return Json(new { Error = "No reason" }, JsonRequestBehavior.AllowGet);
            else
            {
                if (Session["token"] != null)
                    signet.SetToken(Session["token"].ToString());
                try
                {
                    if (string.IsNullOrWhiteSpace(getClaimValue("mobilephone")))
                    {
                        string token = signet.SignDocument(docid, getClaimValue("name"), getClaimValue("mobilephone"),
                            Request.Url.AbsoluteUri.ToLower().Replace("/signdocument", ""));
                        if (string.IsNullOrWhiteSpace(token))
                            return Json(new { error = "Could not reject" }, JsonRequestBehavior.AllowGet);
                        else
                        {
                            Session["token"] = signet.GetToken();
                            return Json(new { token = token, success = true }, JsonRequestBehavior.AllowGet);
                        }
                    }
                    else
                    {
                        bool success = signet.RejectDocument(docid, getClaimValue("name"), getClaimValue("mobilephone"), reason);
                        if (success)
                        {
                            Session["token"] = signet.GetToken();
                            return Json(new { success = true }, JsonRequestBehavior.AllowGet);
                        }
                        else
                            return Json(new { error = "Could not reject" }, JsonRequestBehavior.AllowGet);
                    }
                }
                catch (Exception ex)
                {
                    return Json(new { error = ex.Message }, JsonRequestBehavior.AllowGet);
                }
            }
        }

        [Authorize]
        public ActionResult DeleteDocument(string docid)
        {
            if (string.IsNullOrWhiteSpace(docid))
                return Json(new { Error = "No doc ID" }, JsonRequestBehavior.AllowGet);
            else
            {
                if (Session["token"] != null)
                    signet.SetToken(Session["token"].ToString());
                try
                {
                    bool success = signet.DeleteDocument(docid, getClaimValue("name"), getClaimValue("mobilephone"));
                    if (success)
                    {
                        Session["token"] = signet.GetToken();
                        return Json(new { success = true }, JsonRequestBehavior.AllowGet);
                    }
                    else
                        return Json(new { error = "Could not delete" }, JsonRequestBehavior.AllowGet);
                }
                catch (Exception ex)
                {
                    return Json(new { error = ex.Message }, JsonRequestBehavior.AllowGet);
                }
            }
        }

        [Authorize]
        public ActionResult AddDocument(AddDocumentRequest request)
        {
            if (request == null)
                return Json(new { error = "Invalid request" }, JsonRequestBehavior.AllowGet);
            else
            {
                if (Session["token"] != null)
                    signet.SetToken(Session["token"].ToString());
                try
                {
                    string id = signet.AddDocument(request, getClaimValue("name"), getClaimValue("mobilephone"));
                    if (!string.IsNullOrWhiteSpace(id))
                    {
                        Session["token"] = signet.GetToken();
                        return Json(new { success = true }, JsonRequestBehavior.AllowGet);
                    }
                    else
                        return Json(new { error = "Could not add document" }, JsonRequestBehavior.AllowGet);
                }
                catch (Exception ex)
                {
                    return Json(new { error = ex.Message }, JsonRequestBehavior.AllowGet);
                }
            }
        }

        [Authorize]
        public string GetName()
        {
            return getClaimValue("givenname");
        }

        [AllowAnonymous]
        public ActionResult Login()
        {
            ViewBag.url = ConfigurationManager.AppSettings.Get("IslyklarUrl") + "&qaa=4";
            return View();
        }

        [AllowAnonymous]
        public ActionResult Logout()
        {
            var ctx = Request.GetOwinContext();
            Session["token"] = null;
            var authenticationManager = ctx.Authentication;
            authenticationManager.SignOut(DefaultAuthenticationTypes.ApplicationCookie);
            return RedirectToAction("Login");
        }

        [AllowAnonymous]
        public ActionResult Return(string token)
        {
            try
            {
                var attributes = IslyklarHelper.VerifyIslyklar(token);
                string ssn = attributes.Where(i => i.Key == "userssn").FirstOrDefault().Value;
                string name = attributes.Where(i => i.Key == "name").FirstOrDefault().Value;
                Session["name"] = name;
                string phone = attributes.Where(i => i.Key == "mobile").FirstOrDefault().Value;

                var claims = new List<Claim>();
                claims.Add(new Claim(ClaimTypes.Name, ssn));
                claims.Add(new Claim(ClaimTypes.GivenName, name));
                if (!string.IsNullOrWhiteSpace(phone))
                {
                    claims.Add(new Claim(ClaimTypes.MobilePhone, phone));
                    claims.Add(new Claim(ClaimTypes.Authentication, "Mobile"));
                }
                else
                    claims.Add(new Claim(ClaimTypes.Authentication, "Card"));
                var userID = new ClaimsIdentity(claims, DefaultAuthenticationTypes.ApplicationCookie);

                var ctx = Request.GetOwinContext();
                var authenticationManager = ctx.Authentication;

                if (!authenticationManager.User.Identity.IsAuthenticated)
                {
                    // If not authenticated, then sign in
                    authenticationManager.SignIn(new AuthenticationProperties() { IsPersistent = true }, userID);
                }
                return RedirectToAction("Index");
            }
            catch (Exception ex)
            {
                ViewBag.message = ex.Message;
                ViewBag.stacktrace = ex.StackTrace;
            }
            return View();
        }

        private string getClaimValue(string name)
        {
            var identity = (ClaimsIdentity)User.Identity;
            if (name.ToLower() == "name")
                return identity.Name;
            IEnumerable<Claim> claims = identity.Claims;
            string value = "";
            foreach (Claim claim in claims)
            {
                var type = claim.Type;
                if (claim.Type.ToLower().EndsWith(name))
                    value = claim.Value;
            }
            return value;
        }
    }
}