﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Linq;
using System.Security.Cryptography.X509Certificates;
using System.Security.Cryptography.Xml;
using System.Text;
using System.Web;
using System.Xml;

namespace SignetTokenTestWeb.Helpers
{
    public class IslyklarHelper
    {
        private static string IslyklarSAMLSSN = ConfigurationManager.AppSettings.Get("IslyklarSAMLSSN") ?? "6503760649";

        public static List<KeyValuePair<string, string>> VerifyIslyklar(string token)
        {
            if (string.IsNullOrWhiteSpace(token))
                throw new Exception("Token is null");
            string saml = Encoding.UTF8.GetString(System.Convert.FromBase64String(token));
            XmlDocument doc = new XmlDocument();
            doc.LoadXml(saml);
            SignedXml signedXml = new SignedXml(doc);
            XmlNodeList nodeList = doc.GetElementsByTagName("Signature");
            XmlNodeList certList = doc.GetElementsByTagName("X509Certificate");
            signedXml.LoadXml((XmlElement)nodeList[0]);
            byte[] certData = Encoding.Default.GetBytes(certList[0].InnerText);
            X509Certificate2 cert = new X509Certificate2(certData);
            //CryptoConfig.AddAlgorithm(typeof(RSAPKCS1SHA256SignatureDescription), "http://www.w3.org/2001/04/xmldsig-more#rsa-sha256");
            if (signedXml.CheckSignature(cert, true))
            {
                if (cert.Verify())
                {
                    if (GetCertSSN(cert) == IslyklarSAMLSSN)
                    {
                        DateTime nowTime = DateTime.UtcNow;
                        XmlNodeList conditionList = doc.GetElementsByTagName("Conditions");
                        DateTime fromTime = DateTime.Parse(conditionList[0].Attributes["NotBefore"].Value);
                        DateTime toTime = DateTime.Parse(conditionList[0].Attributes["NotOnOrAfter"].Value);
                        if (nowTime > fromTime && toTime > nowTime)
                        {
                            XmlNodeList attrList = doc.GetElementsByTagName("Attribute");
                            if (attrList.Count > 0)
                            {
                                List<KeyValuePair<string, string>> attributes = new List<KeyValuePair<string, string>>();
                                foreach (XmlNode node in attrList)
                                {
                                    string attributeName = node.Attributes["Name"].Value.ToLower();
                                    string value = node.FirstChild.InnerText;
                                    attributes.Add(new KeyValuePair<string, string>(attributeName, value));
                                }
                                return attributes;
                            }
                            else
                                throw new Exception("No attributes in SAML");
                        }
                        else
                            throw new Exception("Invalid validity");
                    }
                    else
                        throw new Exception("Incorrect SAML certificate");
                }
                else
                    throw new Exception("Invalid certificate");
            }
            else
                throw new Exception("Invalid signature");
        }

        private static string GetCertSSN(X509Certificate2 cert)
        {
            if (cert == null)
                return null;
            else
            {
                string[] subject = cert.Subject.Split(',');
                for (int i = 0; i < subject.Length; i++)
                {
                    string[] attribute = subject[i].Split('=');
                    if (attribute[0].Trim().Equals("SERIALNUMBER") || attribute[0].Trim().ToLower().Equals("oid.2.5.4.5"))
                        return attribute[1];
                }
            }
            return null;
        }
    }
}