﻿using System;
using System.Collections.Generic;
using System.IdentityModel.Policy;
using System.IdentityModel.Protocols.WSTrust;
using System.IdentityModel.Tokens;
using System.IdentityModel.Tokens.Jwt;
using System.IO;
using System.Linq;
using System.Security.Claims;
using System.Security.Cryptography.X509Certificates;
using System.Text;
using System.Web;
using System.Xml;

namespace SignetTokenTestWeb.Helpers
{
    public class TokenManager
    {
        private string _appliesTo;
        private string _issuer;
        private string _cert;
        private string _password;
        private string path = AppDomain.CurrentDomain.BaseDirectory + "/Resources/";

        public TokenManager(string appliesTo, string issuer, string cert, string password)
        {
            _appliesTo = appliesTo;
            _issuer = issuer;
            _cert = cert;
            _password = password;
        }

        public string CreateSaml(string account)
        {
            ClaimsIdentity identity = new
            ClaimsIdentity(AuthenticationTypes.Federation);
            identity.AddClaim(new Claim(ClaimTypes.NameIdentifier, account));
            X509Certificate2 signingCert = new
            X509Certificate2(path + _cert, _password);
            var idpToken = this.MakeToken(identity, signingCert);
            var gIdpToken = idpToken as GenericXmlSecurityToken;
            var assertion = gIdpToken.TokenXml.OuterXml;
            return Convert.ToBase64String(Encoding.UTF8.GetBytes(assertion));
        }

        public DateTime GetValidity(string token)
        {
            DateTime valid = DateTime.Now;

            var tokenHandler = new JwtSecurityTokenHandler();
            try
            {
                var _token = tokenHandler.ReadJwtToken(token);
                valid = _token.ValidTo;
            }
            catch (Exception ex)
            {
                throw new Exception("Could not read token", ex);
            }
            return valid;
        }

        private GenericXmlSecurityToken MakeToken(ClaimsIdentity identity, X509Certificate2 signingCert)
        {
            var descriptor = new SecurityTokenDescriptor
            {
                AppliesToAddress = _appliesTo,
                TokenIssuerName = _issuer,
                SigningCredentials = new
                X509SigningCredentials(signingCert),
                Lifetime = new Lifetime(DateTime.UtcNow.AddMinutes(-5), 
                DateTime.UtcNow.AddHours(1)),
                Subject = identity,
                TokenType = "urn:oasis:names:tc:SAML:2.0:assertion"
            };
            SecurityTokenHandlerCollection tokenHandler = SecurityTokenHandlerCollection.CreateDefaultSecurityTokenHandlerCollection();
            SecurityToken token = tokenHandler.CreateToken(descriptor) as Saml2SecurityToken;
            var outputTokenString = TokenToString(token);
            var handler = new Saml2SecurityTokenHandler();
            var ar = handler.CreateSecurityTokenReference(token, true);
            var uar = handler.CreateSecurityTokenReference(token, false);
            var xmlToken = new GenericXmlSecurityToken(
            GetElement(outputTokenString),
            null,
            DateTime.UtcNow,
            DateTime.UtcNow.AddHours(1),
            ar,
            uar,
            new
            System.Collections.ObjectModel.ReadOnlyCollection<IAuthorizationPolicy>(new
            List<IAuthorizationPolicy>()));
            return xmlToken;
        }
        private string TokenToString(SecurityToken token)
        {
            var genericToken = token as GenericXmlSecurityToken;
            if (genericToken != null)
            {
                return genericToken.TokenXml.OuterXml;
            }
            var handler =
            SecurityTokenHandlerCollection.CreateDefaultSecurityTokenHandlerCollection
            ();
            if (handler.CanWriteToken(token))
            {
                var sb = new StringBuilder(128);
                handler.WriteToken(new XmlTextWriter(new StringWriter(sb)),
                token);
                return sb.ToString();
            }
            else
            {
                throw new InvalidOperationException("Token type not supported");
            }
        }
        private SymmetricProofDescriptor CreateProofDescriptor(X509Certificate2 encryptingCertificate)
        {
            return new SymmetricProofDescriptor(
            256,
            new X509EncryptingCredentials(encryptingCertificate));
        }
        private XmlElement GetElement(string xml)
        {
            XmlDocument doc = new XmlDocument();
            doc.LoadXml(xml);
            return doc.DocumentElement;
        }
    }
}